@@ -137,6 +137,12 @@ class TestBOM(TransactionTestCase):
         response = self.client.post(reverse('bom:part-revision-export-bom-flat', kwargs={'part_revision_id': p1.latest().id}))
         self.assertEqual(response.status_code, 200)
 
+    def test_export_parts(self):
+        (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
+
+        response = self.client.post(reverse('bom:home'), {'download': ''}, follow=True)
+        self.assertEqual(response.status_code, 200)
+
     def test_part_upload_bom(self):
         (p1, p2, p3, p4) = create_some_fake_parts(organization=self.organization)
 
@@ -234,7 +240,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': new_part_mpn,
             'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '',
             'number_variation': '',
             'configuration': 'W',
@@ -260,7 +266,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': 'STM32F401',
             'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '9999',
             'description': 'IC, MCU 32 Bit',
             'revision': 'A',
@@ -276,7 +282,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': '',
             'manufacturer': '',
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '',
             'number_variation': '',
             'description': 'IC, MCU 32 Bit',
@@ -290,7 +296,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': '',
             'manufacturer': '',
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '1234',
             'description': 'IC, MCU 32 Bit',
             'revision': 'A',
@@ -306,7 +312,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': '',
             'manufacturer': '',
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '1235',
             'number_variation': '',
             'description': 'IC, MCU 32 Bit',
@@ -321,7 +327,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': 'ABC123',
             'manufacturer': '',
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '',
             'number_variation': '',
             'description': 'IC, MCU 32 Bit',
@@ -345,7 +351,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': new_part_mpn,
             'manufacturer': p1.primary_manufacturer_part.manufacturer.id,
-            'number_class': p1.number_class.id,
+            'number_class': (p1.number_class),
             'number_item': '2000',
             'number_variation': '01',
             'configuration': 'W',
@@ -375,7 +381,7 @@ class TestBOM(TransactionTestCase):
         new_part_form_data = {
             'manufacturer_part_number': '',
             'manufacturer': '',
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '2000',
             'configuration': 'W',
             'description': 'IC, MCU 32 Bit',
@@ -400,7 +406,7 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 200)
 
         edit_part_form_data = {
-            'number_class': p1.number_class.id,
+            'number_class': str(p1.number_class),
             'number_item': '',
             'number_variation': '',
         }
@@ -522,7 +528,6 @@ class TestBOM(TransactionTestCase):
     def test_upload_parts(self):
         create_some_fake_part_classes(self.organization)
 
-        # part_count = Part.objects.all().count()
         # Should pass
         with open('bom/test_files/test_new_parts.csv') as test_csv:
             response = self.client.post(reverse('bom:upload-parts'), {'file': test_csv}, follow=True)
@@ -532,6 +537,10 @@ class TestBOM(TransactionTestCase):
         new_part_count = Part.objects.all().count()
         self.assertEqual(new_part_count, 4)
 
+        # Part revs should be created for each part
+        for p in Part.objects.all():
+            self.assertIsNotNone(p.latest())
+
         # Should fail because class doesn't exist
         with open('bom/test_files/test_new_parts_2.csv') as test_csv:
             response = self.client.post(reverse('bom:upload-parts'), {'file': test_csv})
@@ -552,6 +561,18 @@ class TestBOM(TransactionTestCase):
                 found_error = True
         self.assertTrue(found_error)
 
+    def test_upload_parts_break_tolerance(self):
+        create_some_fake_part_classes(self.organization)
+
+        # Should break with data error
+        with open('bom/test_files/test_new_parts_broken.csv') as test_csv:
+            response = self.client.post(reverse('bom:upload-parts'), {'file': test_csv}, follow=True)
+        messages = list(response.context.get('messages'))
+
+        self.assertTrue(len(messages) > 0)
+        for msg in messages:
+            self.assertEqual(msg.tags, 'error')
+
     def test_upload_part_classes(self):
         # Should pass
         with open('bom/test_files/test_part_classes.csv') as test_csv:
@@ -579,6 +600,14 @@ class TestBOM(TransactionTestCase):
         self.assertTrue('in row 3 does not have a value. Uploading of this part class skipped.' in str(response.content))
         self.assertTrue('in row 4 does not have a value. Uploading of this part class skipped.' in str(response.content))
 
+        # Submit with a csv file exported with a byte order mask, typically from MS word I think
+        with open('bom/test_files/test_part_classes_byte_order.csv') as test_csv:
+            response = self.client.post(reverse('bom:settings'), {'file': test_csv, 'submit-part-class-upload': ''}, follow=True)
+        self.assertEqual(response.status_code, 200)
+        messages = list(response.context.get('messages'))
+        for msg in messages:
+            self.assertTrue('None on row' not in str(msg.message))
+
     def test_upload_part_classes_parts_and_boms(self):
         self.organization.number_item_len = 5
         self.organization.save()
@@ -599,6 +628,8 @@ class TestBOM(TransactionTestCase):
         self.assertEqual(response.status_code, 200)
         new_part_count = Part.objects.all().count()
         self.assertEqual(new_part_count, 88)
+        for p in Part.objects.all():
+            self.assertIsNotNone(p.latest())
 
         pcba_class = PartClass.objects.filter(code=652).first()
         pcba = Part.objects.filter(number_class=pcba_class, number_item='00003', number_variation='0A').first()
@@ -1074,7 +1105,7 @@ class TestBOMIntelligent(TestBOM):
         subparts = pcba.latest().assembly.subparts.all().order_by('id')
         self.assertEqual(subparts[0].reference, 'C1, C2, C3')
         self.assertEqual(subparts[1].reference, 'C4, C5')
-        self.assertEqual(subparts[2].reference, None)
+        self.assertEqual(subparts[2].reference, '')
 
 
 class TestBOMNoVariation(TestBOM):
@@ -1120,7 +1151,7 @@ class TestForms(TestCase):
     def test_part_form(self):
         (pc1, pc2, pc3) = create_some_fake_part_classes(self.organization)
         form_data = {
-            'number_class': pc1.id,
+            'number_class': str(pc1),
             'description': "ASSY, ATLAS WRISTBAND 10",
             'revision': 'AA'
         }
@@ -1131,7 +1162,7 @@ class TestForms(TestCase):
         (m1, m2, m3) = create_some_fake_manufacturers(self.organization)
 
         form_data = {
-            'number_class': pc2.id,
+            'number_class': str(pc2),
             'description': "ASSY, ATLAS WRISTBAND 5",
             'revision': '1',
         }
