@@ -31,7 +31,7 @@ from bom.forms import PartInfoForm, PartFormSemiIntelligent, PartFormIntelligent
     PartRevisionNewForm, PartCSVForm, PartClassForm, PartClassSelectionForm, PartClassCSVForm, UploadBOMForm, BOMCSVForm, PartClassFormSet, \
     OrganizationCreateForm, OrganizationFormEditSettings
 from bom.utils import listify_string, stringify_list, check_references_for_duplicates, prep_for_sorting_nicely
-from bom.csv_headers import PartsListCSVHeaders, PartClassesCSVHeaders, BOMFlatCSVHeaders, BOMIndentedCSVHeaders
+from bom.csv_headers import PartsListCSVHeaders, PartsListCSVHeadersSemiIntelligent, PartClassesCSVHeaders, BOMFlatCSVHeaders, BOMIndentedCSVHeaders
 
 logger = logging.getLogger(__name__)
 
@@ -62,7 +62,7 @@ def home(request):
         # Note that posting a PartClass selection does not include a named parameter in
         # the POST, so this case is the de facto "else" clause.
     else:
-        part_class_selection_form = PartClassSelectionForm(request.GET, organization=organization)
+        part_class_selection_form = PartClassSelectionForm(organization=organization)
 
     if part_class_selection_form.is_valid():
         part_class = part_class_selection_form.cleaned_data['part_class']
@@ -164,22 +164,35 @@ def home(request):
     if 'download' in request.POST:
         response = HttpResponse(content_type='text/csv')
         response['Content-Disposition'] = 'attachment; filename="indabom_parts_search.csv"'
-        csv_headers = PartsListCSVHeaders()
+        csv_headers = organization.part_list_csv_headers()
         writer = csv.DictWriter(response, fieldnames=csv_headers.get_default_all())
         writer.writeheader()
         for part_rev in part_revs:
-            row = {
-                csv_headers.get_default('part_number'): part_rev.part.full_part_number(),
-                csv_headers.get_default('part_category'): part_rev.part.number_class.name,
-                csv_headers.get_default('part_revision'): part_rev.revision,
-                csv_headers.get_default('part_manufacturer'): part_rev.part.primary_manufacturer_part.manufacturer.name if part_rev.part.primary_manufacturer_part is not None and
-                                                                                                  part_rev.part.primary_manufacturer_part.manufacturer is not None else '',
-                csv_headers.get_default('part_manufacturer_part_number'): part_rev.part.primary_manufacturer_part.manufacturer_part_number if part_rev.part.primary_manufacturer_part is not None else '',
-            }
-            for field_name in csv_headers.get_default_all():
-                if field_name not in csv_headers.get_defaults_list(['part_number', 'part_category', 'part_synopsis', 'part_revision', 'part_manufacturer', 'part_manufacturer_part_number', ]):
-                    attr = getattr(part_rev, field_name)
-                    row.update({csv_headers.get_default(field_name): attr if attr is not None else ''})
+            if organization.number_scheme == constants.NUMBER_SCHEME_SEMI_INTELLIGENT:
+                row = {
+                    csv_headers.get_default('part_number'): part_rev.part.full_part_number(),
+                    csv_headers.get_default('part_category'): part_rev.part.number_class.name,
+                    csv_headers.get_default('part_revision'): part_rev.revision,
+                    csv_headers.get_default('part_manufacturer'): part_rev.part.primary_manufacturer_part.manufacturer.name if part_rev.part.primary_manufacturer_part is not None and
+                                                                                                      part_rev.part.primary_manufacturer_part.manufacturer is not None else '',
+                    csv_headers.get_default('part_manufacturer_part_number'): part_rev.part.primary_manufacturer_part.manufacturer_part_number if part_rev.part.primary_manufacturer_part is not None else '',
+                }
+                for field_name in csv_headers.get_default_all():
+                    if field_name not in csv_headers.get_defaults_list(['part_number', 'part_category', 'part_synopsis', 'part_revision', 'part_manufacturer', 'part_manufacturer_part_number', ]):
+                        attr = getattr(part_rev, field_name)
+                        row.update({csv_headers.get_default(field_name): attr if attr is not None else ''})
+            else:
+                row = {
+                    csv_headers.get_default('part_number'): part_rev.part.full_part_number(),
+                    csv_headers.get_default('part_revision'): part_rev.revision,
+                    csv_headers.get_default('part_manufacturer'): part_rev.part.primary_manufacturer_part.manufacturer.name if part_rev.part.primary_manufacturer_part is not None and
+                                                                                                      part_rev.part.primary_manufacturer_part.manufacturer is not None else '',
+                    csv_headers.get_default('part_manufacturer_part_number'): part_rev.part.primary_manufacturer_part.manufacturer_part_number if part_rev.part.primary_manufacturer_part is not None else '',
+                }
+                for field_name in csv_headers.get_default_all():
+                    if field_name not in csv_headers.get_defaults_list(['part_number', 'part_synopsis', 'part_revision', 'part_manufacturer', 'part_manufacturer_part_number', ]):
+                        attr = getattr(part_rev, field_name)
+                        row.update({csv_headers.get_default(field_name): attr if attr is not None else ''})
             writer.writerow({k: smart_str(v) for k, v in row.items()})
         return response
 
@@ -744,8 +757,7 @@ def export_part_list(request):
         'part_manufacturer_part_number',
     ]
 
-    csv_headers = PartsListCSVHeaders()
-
+    csv_headers = organization.part_list_csv_headers()
     writer = csv.DictWriter(response, fieldnames=fieldnames)
     writer.writeheader()
     for item in parts:
@@ -780,6 +792,11 @@ def create_part(request):
 
     PartForm = PartFormSemiIntelligent if organization.number_scheme == constants.NUMBER_SCHEME_SEMI_INTELLIGENT else PartFormIntelligent
 
+    if organization.number_scheme == constants.NUMBER_SCHEME_SEMI_INTELLIGENT and PartClass.objects.count() == 0:
+        messages.info(request, f'Welcome to IndaBOM! Before you create your first part, you must create your first part class. '
+                               f'<a href="{reverse("bom:help")}#part-numbering" target="_blank">What is a part class?</a>')
+        return HttpResponseRedirect(reverse('bom:settings', kwargs={'tab_anchor': 'indabom'}))
+
     if request.method == 'POST':
         part_form = PartForm(request.POST, organization=organization)
         manufacturer_form = ManufacturerForm(request.POST)
